# script to fit curves to article accumulation data
library(AICcmodavg)
library(boot)


data <- read.csv("timelines_data.csv",
  stringsAsFactors = FALSE)
# str(data)

# reorganise to long format
data_list <- as.list(seq_len(ncol(data)-2))
for(i in 1:(ncol(data)-2)){
  result <- data[, c(1, 2, (i + 2))]
  result$year <- as.numeric(substr(colnames(result)[3], 2, 5))
  colnames(result)[3] <- "count"
  data_list[[i]] <- result[, c(1, 2, 4, 3)]
}
data_clean <- as.data.frame(do.call(rbind, data_list))
data_clean$review_type <- "a) Systematic review"
data_clean$review_type[grepl("M", data_clean$id)] <- "b) Systematic map"


# plot on a linear scale
ggplot(data_clean, aes(x = year, y = count, group = id)) +
  geom_line() +
  xlim(c(1980, 2020)) +
  ylim(c(0, 8000)) +
  ylab("Number of Articles Published") +
  xlab("Year") +
  facet_wrap(~ review_type) +
  theme_bw() +
  theme(
    strip.background = element_blank(),
    strip.text = element_text(hjust = 0)
  )
ggsave("updated_count_plot.pdf")

# or for log scale
data_clean$count_min <- data_clean$count
data_clean$count_min[data_clean$count_min < 1] <- 0.1

ggplot(data_clean, aes(x = year, y = count_min, group = id)) +
  scale_y_continuous(trans = "log10") +
  geom_line() +
  xlim(c(1980, 2020)) +
  ylab("Number of Articles Published") +
  xlab("Year") +
  facet_wrap(~ review_type) +
  theme_bw() +
  theme(
    strip.background = element_blank(),
    strip.text = element_text(hjust = 0)
  )


#data_clean$year_scaled <- scale(data_clean$year)
#
# data_clean$year_scaled <- data_clean$year - mean(unique(data_clean$year))

# or range from 0 -1
# data_clean$year_scaled <- data_clean$year - min(data_clean$year)
# data_clean$year_scaled <- data_clean$year_scaled / max(data_clean$year_scaled)
# data_clean$year_scaled <- data_clean$year_scaled + 1


# split by review
data_split <- split(data_clean, data_clean$id)

# remove reviews or maps with all NA values
remove_vals <- unlist(lapply(data_split, function(a){
  all(is.na(a$count))
}))
data_split <- data_split[which(!remove_vals)]
data_clean <- do.call(rbind, data_split)


# TRY MODELLING ONE DATASET
data_temp <- data_split[[1]]

# start with mgcv
library(mgcv)
# model_mgcv <- mgcv::gam(count ~ s(year),
#   family = poisson(link = "log"),
#   data = data_temp)
model_mgcv <- mgcv::gam(count ~ s(year),
  family = gaussian(link = "identity"),
  data = data_temp)

plot(predict(model_mgcv) ~ year, data = data_temp)
# plot(exp(predict(model_mgcv)) ~ year, data = data_temp)

data_temp$est <- predict(model_mgcv)
par(mar = c(4, 5, 1, 1))
plot(count ~ year,
  data = data_temp,
  las = 1,
  bty = "l",
  ann = FALSE,
  pch = 20,
  col = "grey"
)
mtext("Year", 1, line = 2.5)
mtext("Number of Articles", 2, line = 3.5)
lines(est ~ year, data = data_temp)

# plot rate of change (first derivative)
test <- Deriv(model_mgcv)$year
test$year <- seq(
  min(data_temp$year),
  max(data_temp$year),
  length.out = 200
)
plot(deriv ~ year, data = test)


# apply this to whole dataset
model_list <- lapply(data_split, function(a){
  mgcv::gam(count ~ s(year),
    family = gaussian(link = "identity"),
    data = a)
})

model_list_poisson <- lapply(data_split, function(a){
  mgcv::gam(count ~ s(year),
    family = poisson(link = "log"),
    data = a)
})

# extract & plot predictions
data_list <- lapply(seq_len(length(data_split)), function(a, model, data){
  data[[a]]$predicted_n <- as.numeric(predict(model[[a]], type = "response"))
  data[[a]]$predicted_rate <- as.numeric(
    Deriv(model[[a]], newdata = data.frame(year = data[[a]]$year))$year$deriv
  )
  return(data[[a]])
},
model = model_list,
data = data_split
)

predictions <- do.call(rbind, data_list)
predictions$review_number <- as.factor(
  as.numeric(substr(predictions$id, 3, nchar(predictions$id)))
)
predictions$review_type <- "a) Systematic review"
predictions$review_type[which(substr(predictions$id, 2, 2) == "M")] <- "b) Systematic map"
# plot(predicted_n ~ year, data = data_list[[1]])
# plot(predicted_rate ~ year, data = data_list[[1]])


# plot with ggplot
library(ggplot2)
library(scales)

ggplot(predictions,
  aes(x = year, y = predicted_n, group = review_number)) +
  # scale_y_continuous(trans = "log") + # try on log scale
  geom_line() +
  ylab("Number of Articles Published") +
  xlab("Year") +
  facet_wrap(~ review_type) +
  theme_bw() +
  theme(
    strip.background = element_blank(),
    strip.text = element_text(hjust = 0)
  )

ggplot(predictions,
  aes(x = year, y = predicted_rate, color = review_number)) +
  geom_line() +
  ylab("Publication Rate (1st derivative of GAM smooth)") +
  xlab("Year") +
  facet_wrap(~ review_type)


# are those with peak rates same as with peak values
data_max <- lapply(data_list, function(a){
  data.frame(
    id = a$id[1],
    rate = max(a$predicted_rate),
    count = max(a$count),
    stringsAsFactors = FALSE
  )
})
data_max <- do.call(rbind, data_max)

ggplot(data_max,
  aes(x = count, y = rate, label = id)
)+ geom_text() +
scale_y_log10() + scale_x_log10() +
xlab("Maximum Article Count") +
ylab("Maximum Publication Rate")


# use gams?
library(gam)
model_gam <- gam::gam(log(count + 1) ~ s(year, df = 6), data = data_temp)
model_gam <- gam::gam(count ~ s(year, df = 6),
  family = poisson(link = "log"),
  data = data_temp)
# note: only works if mgcv not called first
# weirdly, specifying gam::s or VGAM::s (the only options) all lead to linear fits
# summary(model_gam)
# new_data <- data.frame(
#   year = c(min(data_temp$year):max(data_temp$year)))
# new_data$prediction <- predict(model_gam, new_data)
# investigate:
plot(predict(model_gam) ~ year, data = data_temp)
data_temp$est <- exp(predict(model_gam)) - 1
par(mar = c(4, 5, 1, 1))
plot(count ~ year,
  data = data_temp,
  las = 1,
  bty = "l",
  ann = FALSE,
  pch = 20,
  col = "grey"
)
mtext("Year", 1, line = 2.5)
mtext("Number of Articles", 2, line = 3.5)
lines(est ~ year, data = data_temp)



# TRY MODELLING ONE DATASET
data_temp <- data_split[[1]]# [

#  which(cumsum(data_split[[1]]$count) > 0), ]

model_lm <- lm(logit((count + 1)/max(count + 2)) ~ year_scaled, data = data_temp)

nls_control <- nls.control(
  maxiter = 10^4,
  minFactor = 10^-6,
  warnOnly = TRUE
)

# model types
model_lm <- nls(
  count ~ a + b * year_scaled,
  start = list(a = 1, b = 1),
  data = data_split[[1]]
)

# model_power <- nls(
#   count ~  a + b * year_scaled ^ c,
#   start = list(a = 1, b = 1, c = 1),
#   data = data_clean,
#   control = nls_control
# )
data_temp <- data_split[[1]][
  which(cumsum(data_split[[1]]$count) > 0), ]

model_power <- nls(
  count ~  b * year_scaled ^ c,
  start = list(b = 1, c = 1),
  data = data_temp, #data_split[[1]],
  control = nls_control
)
# model_power <- nls(
#   I(count + 1) ~  year_scaled ^ c,
#   start = list(c = 2),
#   data = data_clean,
#   control = nls_control
# )

model_logit <- nls(
  count ~ a / (1 + exp((b - year_scaled) * k)),
  start = list(a = 1000, b = 1, k = 1),
  data = data_temp,
  control = nls_control
)

# calculate best model using AICc
aictab(list(
  lm = model_lm,
  power = model_power,
  logit = model_logit
))



par(mar = c(4, 5, 1, 1))
plot(count ~ year_scaled,
  data = data_split[[1]],
  las = 1,
  bty = "l",
  ann = FALSE,
  pch = 20,
  col = "grey"
)
mtext("Year (starting 1945)", 1, line = 2.5)
mtext("Number of Articles", 2, line = 3.5)
mtext(data_split[[1]]$id[1], 3, line = -2, font = 2, adj = 0.1)
lines(
  x = data_split[[1]]$year_scaled,
  y = predict(
    model_lm,
    newdata = data.frame(year_scaled = data_split[[1]]$year_scaled)
  )
)
lines(
  x = data_split[[1]]$year_scaled,
  y = predict(
    model_power,
    newdata = data.frame(year_scaled = data_split[[1]]$year_scaled)
  ),
  col = "red"
)
lines(
  x = data_split[[1]]$year_scaled,
  y = predict(
    model_logit,
    newdata = data.frame(year_scaled = data_split[[1]]$year_scaled)
  ),
  col = "blue"
)



# summaries
summary(model_power)
summary(model_logit)
summary(model_lm)



## TRY THIS FOR ALL DATASETS AT ONCE
nls_control <- nls.control(
  maxiter = 10^4,
  minFactor = 10^-10,
  warnOnly = TRUE
)
# model_list <- lapply(data_split, function(x){
# return(list(
#   linear = model_lm,
#   power = model_power,
#   logit = model_logit
# ))
# })

model_list <- list()
for(i in 1:length(data_split)){
  x <- data_split[[i]]
  model_lm <- nls(
    count ~ a + b * year_scaled,
    start = coef(model_lm),
    data = x
  )
  model_power_lm <- glm(
    count ~ year_scaled,
    data = x
  )
  model_power <- nls(
    count ~  b * exp(year_scaled * c),
    start = c(
      b = coef(model_power_lm)[[1]],
      c = exp(coef(model_power_lm)[[2]])
    ),
    data = x,
    control = nls_control
  )
  model_logit <- nls(
    count ~ a / (1 + exp((b - year_scaled) * k)),
    start = c(
      a = max(x$count) * 2,
      b = 70,
      k = 1
    ),
    data = x,
    control = nls_control
  )
  model_list[[i]] <- list(
    linear = model_lm,
    power = model_power,
    logit = model_logit
  )
}



par(mar = c(4, 5, 1, 1))
plot(count ~ year_scaled,
  data = data_split[[2]],
  las = 1,
  bty = "l",
  ann = FALSE,
  pch = 20,
  col = "grey"
)
mtext("Year (starting 1945)", 1, line = 2.5)
mtext("Number of Articles", 2, line = 3.5)
mtext(data_split[[2]]$id[1], 3, line = -2, font = 2, adj = 0.1)
lines(
  x = data_split[[2]]$year_scaled,
  y = predict(
    model_list[[2]]$linear,
    newdata = data.frame(year_scaled = data_split[[2]]$year_scaled)
  )
)
lines(
  x = data_split[[2]]$year_scaled,
  y = predict(
    model_list[[2]]$power,
    newdata = data.frame(year_scaled = data_split[[2]]$year_scaled)
  ),
  col = "red"
)
lines(
  x = data_split[[2]]$year_scaled,
  y = predict(
    model_list[[2]]$logit,
    newdata = data.frame(year_scaled = data_split[[2]]$year_scaled)
  ),
  col = "blue"
)


aictab(model_list[[2]])


